#!/usr/bin/env python
"""URL definitions for GRR Admin Server."""


import mimetypes


from django.conf import urls
from django.views.static import serve

from grr.lib import config_lib
from grr.lib import registry
import grr.gui

django_base = "django."
view_base = "grr.gui.views."
handler404 = "urls.handler404"
handler500 = view_base + "ServerError"
static_handler = serve


def GetURLPatterns():
  return [
      urls.url(r"^$", grr.gui.views.Homepage),
      # Automatic rendering is done here
      urls.url(r"^api/.+", grr.gui.views.RenderApi),
      urls.url(r"^render/[^/]+/.*", grr.gui.views.RenderGenericRenderer),
      urls.url(r"^download/[^/]+/.*", grr.gui.views.RenderBinaryDownload),
      urls.url(r"^static/(.*)$", static_handler, {"document_root": config_lib.CONFIG["AdminUI.document_root"]}),
      urls.url(r"^local/static/(.*)$", static_handler, {"document_root": config_lib.CONFIG["AdminUI.local_document_root"]}),
      urls.url(r"^help/(.*)$", grr.gui.views.RenderHelp)
  ]


urlpatterns = []


class UrlsInit(registry.InitHook):
  pre = []

  def RunOnce(self):
    """Run this once on init."""
    mimetypes.add_type("application/font-woff", ".woff", True)
    urlpatterns.extend(GetURLPatterns())
